/* 
   Steuerung der WaermePumpe
*/

// Taktfrequenz, wird fr Berechnung von 
// delays und Frequenzen bentigt

#ifndef F_CPU
#define F_CPU 16000000
#endif


#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#include "main.h"
#include "lcd.h"


/* Ringpuffer und ADC-relevantes*/
#define ADC_BUFFER_SIZE 64U
#define TEMP_DECIMALS 1					

#define DISPLAY_UPDATE_INTERVAL	32	//Modulo-Wert fr Timer-Interrupt Display Update	(32 fuer jede Sekunde)

/* Statische TEXTE */
char leer[] PROGMEM  = "    ";
char ztext[] PROGMEM = "Ctr:";


/* Steuerung zeitlicher Ablufe */
volatile static uint32_t counterticks;					//Zhlvariable


// Wir zaehlen Tastendruecke
volatile static uint8_t zaehler;
volatile static uint8_t pinstat;




void UpdateDisplay(void);		  //Aktualisiert die 7-Segment Anzeige

/*Steuer- und Konfigurationsregister */
volatile static reg8 statusreg;				//Flags wie folgt BIT_0: UART: CR received
																			//								BIT_1: Command Error
																			//								BIT_2: Parameter Error
																			//								BIT_4: Display Update Required
																			
volatile static reg8 encoderreg;      //Flags wie folgt BIT_4: Zustand Input1
																			//								BIT_5: Zustand Input2
																			//                BIT_6: Fallende Flanke erkannt
																			//                BIT_7: Encoder lesen
																			//              BIT0..3: Zaehler, wie oft Gleichheit detektiert
															

/* Interrupt-Vektoren
   Prioritt (absteigend)  TIMER1OVF  -> TIMER0OVF -> UART(RX) -> UART(UDRE) -> ADC */

ISR(TIMER1_OVF_vect)
{		
    //Retten des SREG nicht ntig, da Timer 1 OVF hhere Prioritt als UART und als ADC
		
		//Zhlvariable auswerten
		if((++counterticks % DISPLAY_UPDATE_INTERVAL) == 0)	//Zeit, Display upzudaten!
				statusreg |=  (1<<BIT_4);		//Flag setzen, dass Display upgedatet werden soll (wird in main() ausgewertet!) 
				
		//neue ADC-Wandlung starten:
		//ADC_convert();
		
		
}

ISR(TIMER0_OVF_vect)
{
    static uint8_t oldval = MASK_BIT_4;
		
		//Kein Request fr Encoder lesen
		if(!(encoderreg & MASK_BIT_7))
			return;

		//Register retten:
		reg8 sreg_tmp;
		sreg_tmp = SREG; 
    cli();
		
		
		//Eingnge samplen:
		uint8_t inbuf = PINC;
		
		//und auswerten:
		if((encoderreg & MASK_BIT_4) == (inbuf & MASK_BIT_4)) // Gleichheit mit letztem Wert
		{
			if((encoderreg & 0x0F) == 0x08) //8mal gleich -> Wert stabil!
			{
				if((inbuf & MASK_BIT_5))
					encoderreg |= (1<<BIT_5);
				else
				  encoderreg &= ~(1<<BIT_5);
				
				//Fallende Flanke erkennen
				if(oldval == MASK_BIT_4)
				{
				  if(!(inbuf & MASK_BIT_4))
					{
						encoderreg |= (1<<BIT_6); //Anzeigen dass fallende Flanke
					}					
				}
				//Wert korrigieren
				oldval = (inbuf & MASK_BIT_4)	;
						
				//Job getan Request-Bit lschen		
				encoderreg &= ~(1<<BIT_7);
			}
			else //Wert noch nicht stabil!
			{
				encoderreg &= ~(1<<BIT_6);
				encoderreg++; //neuer Versuch!
			}
		}
		else  // Werte nicht gleich!
		{
			if(inbuf & MASK_BIT_4)
				encoderreg |= (1<<BIT_4);
			else
			  encoderreg &= ~(1<<BIT_4);			
		  
			encoderreg &= 0xF0; //Zhler lschen
			encoderreg &= ~(1<<BIT_6);		
		}
			
		SREG = sreg_tmp;		

}

/* Vorbereiten der Texte */
void PrepareDisplay(void)
{

		lcd_clear();
		lcd_home();
		lcd_string_P(ztext);
		
}


/* Update des Displays*/
void UpdateDisplay(void)
{
		char text[16];
		static uint8_t active = 0;
		
		//lcd_clear();
		lcd_setcursor(0+4,0);
		lcd_string_P(leer);
		
		
		itoa(zaehler, text, 10);
		lcd_setcursor(0+5,0);
		lcd_string(text);
		//lcd_data(0x01);
		
		/*
		lcd_setcursor(8+3,0);
		lcd_string_P(leer);
		
		itoa(ADCmean1, text, 10);
		lcd_setcursor(8+3,0);
		lcd_string(text);
		lcd_data(0x02);


		lcd_setcursor(0+3,1);
		lcd_string_P(leer);
		
		itoa(ADCmean2, text, 10);
		lcd_setcursor(0+3,1);
		lcd_string(text);
		lcd_data(0x03);
		
		lcd_setcursor(8+3,1);
		lcd_string_P(leer);
		
		itoa(ADCmean3, text, 10);
		lcd_setcursor(8+3,1);
		lcd_string(text);
    */
		
		if(active == 1)
		{
		  lcd_setcursor(15,0);
			lcd_string("*");
			active = 0;
		}
		else
		{
		  lcd_setcursor(15,0);
			lcd_string(" ");
			active = 1;
		}
		
}


/*PROGRAMM*/
int main( void )
{							
	  reg8 encoder;
				
		//Status: Neutral
		statusreg = 0;
		
		//Zaehler reset:
		zaehler = 50;
				
		//Encoder vorbereiten:
		encoderreg = 0b10110000;
				
		//Eingnge setzten:
		DDRC &= ~((1<<PC4) | (1<<PC5));
		//PD0 als Kontrollausgang!
		DDRD |= (1<<PD0);
		

		// LCD initialisieren
		lcd_init();
		PrepareDisplay();
		
		// TIMER1 als Taktgeber (normaler Modus) erzeugt ohne Prescaler alle 4.096ms einen berlauf (@16Mhz)
		counterticks = 0;									// Zhlvariable zurcksetzen;
		TCCR1A = 0b00000000;		
		TCCR1B = CTR_CLKS_PRESCALE_8;			// berlauf alle 32ms	
		//TCCR1B = CTR_CLKS_PRESCALE_1;
		//TCCR0 = CTR_CLKS_PRESCALE_64;			// berlauf jede 1.024ms (nur fr Tastenerkennung!)
		TCCR0 = CTR_CLKS_PRESCALE_1;
		
		TIMSK |=  ( (1<<TOIE1) | (1<<TOIE0));				// Interrupt(s) erlauben
						

		//itoa(encoderreg, text, 10);
		//lcd_setcursor(0,1);
		//lcd_string(text);


		
		// Interrupts global erlauben
		sei();			
		
				

    while(1)
		{
					
				//Zhlerstand korrigieren
				encoder = encoderreg;
												
				if(!(encoder & MASK_BIT_7))   //fertig mit der Arbeit!
				{	
					if(encoder & MASK_BIT_6) //Fallende Flanke erkannt
					{
						if(encoder & MASK_BIT_5)
							zaehler++;
						else
							zaehler--;
							
						encoderreg &= ~(1<<BIT_6); //Flanke wurde bearbeitet!
					}				
					encoderreg |= (1<<BIT_7); //neuen Wert anfordern
				}
				
			
				// Muss Display aktualisiert werden? 
				if(statusreg & MASK_BIT_4)
				{			
						UpdateDisplay();
						statusreg &= ~( (1<<BIT_4) );		//Flag lschen, da Update erfolgt
				}
				
		
		}   							

    return 0;
};

